import { useEffect, useRef, useState } from 'react';
import styles from './Emulator.module.scss';

type EmulatorSystem = (
  'gb' | 'gbc' | 'gba'
);

type EmulatorCore = (
  'gambatte' | 'mgba'
);

type SendEmulatorMessageInit = {
  message:'init';
  core:EmulatorCore;
  system:EmulatorSystem;
  gameId:string;
  gameName:string;
}

type SendEmulatorMessageVolume = {
  message:'volume';
  volume:number;
};

type SendEmulatorMessage = (
  SendEmulatorMessageInit |
  SendEmulatorMessageVolume
);

type ReceiveEmulatorMessage = (
  { message: 'start' } |
  { message: 'ready' } |
  { message: 'save' } |
  { message: 'load' }
);

export type EmulatorProps = {
  system:EmulatorSystem;
};

const EMULATOR_SYSTEM_CORES:{ [key in EmulatorSystem]:EmulatorCore } = {
  'gb': 'gambatte',
  'gbc': 'gambatte',
  'gba': 'mgba'
};

export const Emulator:React.FC<EmulatorProps> = props => {
  const iframeRef = useRef<HTMLIFrameElement>(null);
  const [ hasInit, setHasInit ] = useState(false);

  const send = (msg:SendEmulatorMessage) => {
    iframeRef.current?.contentWindow?.postMessage(msg, '*');
  };

  useEffect(() => {
    if(hasInit) return;
    setHasInit(true);

    send({
      message: 'init',
      core: EMULATOR_SYSTEM_CORES[props.system],
      system: props.system,
      gameId: '0',
      gameName: 'Pokemon - Crystal Version'
    });

    window.onmessage = (e) => {
      if(!e.data) {
        console.error('Invalid message received:', e.data);
        return;
      }
      
      const msg = e.data as ReceiveEmulatorMessage;
      switch(msg.message) {
        case 'start':
        case 'ready':
        case 'load':
        case 'save':
          break;
        default:
          console.error('Invalid message received:', msg);
          break
      }
    };
  }, [ iframeRef ]);

  return (
    <div className={styles.emulator}>
      <div className={[
        styles.emulator__box,
        styles[`emulator__box--${props.system}`]
      ].join(' ')} />

      <iframe
        src="/emulator.html"
        className={styles.emulator__iframe}
        ref={iframeRef}
      />
    </div>
  );
}